/*-
 * Copyright (c) 1999 Daniel Capo Sobral
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: chconf.c,v 1.9 1999/12/19 21:33:03 dcs Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/errno.h>
#include <unistd.h>
#include "libconf.h"

void usage(void);

int
main(int argc, char** argv)
{
	struct configuration*	conf;
	int			quiet = 0;
	int			level = 1;
	int			nlevels = 0;
	int			op;
	char*			buffer;
	int			error;

	if(argc < 4)
		usage();

	conf = createconf(1);

	while(argc > 1) {
		while((op = getopt(argc, argv, "f:F:l:u:q")) != -1)
			switch(op) {
			    case 'f':
				nlevels++;
				if(readconf(conf, optarg) == ENOENT) {
					if(!quiet)
						fprintf(stderr, "Warning: %s not found\n", optarg);
					nlevels--;
				}
				break;
			    case 'F':
				switch(optarg[0]) {
				    case 'R':
					nlevels++;
					if(readconf(conf, "/etc/defaults/rc.conf") == ENOENT) {
						if(!quiet)
							fprintf(stderr, "Warning: /etc/defaults/rc.conf not found");
						nlevels--;
					}
					nlevels++;
					if(readconf(conf, "/etc/rc.conf") == ENOENT)
						nlevels--;
					nlevels++;
					if(readconf(conf, "/etc/rc.conf.local") == ENOENT)
						nlevels--;
					break;
				    case 'L':
					nlevels++;
					if(readconf(conf, "/boot/defaults/loader.conf") == ENOENT) {
						if(!quiet)
							fprintf(stderr, "Warning: /boot/defaults/loader.conf not found");
						nlevels--;
					}
					nlevels++;
					if(readconf(conf, "/boot/loader.conf") == ENOENT)
						nlevels--;
					nlevels++;
					if(readconf(conf, "/boot/loader.conf.local") == ENOENT)
						nlevels--;
					break;
				    default:
					usage();
				}
				break;
			    case 'l':
				level = atoi(optarg);
				break;
			    case 'q':
				quiet = 1;
				break;
			    case 'u':
				unsetvarlevelbyname(conf, level, optarg);
				break;
			    case '?':
			    default:
				usage();
			}

		argc -= optind;
		argv += optind;

		while(*argv && **argv != '-') {
			if(*argv[1] != '"') {
				buffer = malloc(strlen(argv[1]+1));
				sprintf(buffer, "%s", argv[1]);
			} else
				buffer = strdup(argv[1]);
			if(setvarlevelbyname(conf, level, argv[0], buffer) && !quiet)
				fprintf(stderr, "Error setting %s!\n", argv[0]);
			free(buffer);
			argv++;
			argv++;
			argc--;
			argc--;
			/* Debugging stuff
			if((argc/2)%2)
				error = writeconf(conf);
			else
				error = 0;
			if(!quiet && error == 1) {
				fprintf(stderr, "Error committing the changes!\n");
				return 2;
			}
			if(!quiet && error == -1) {
				fprintf(stderr, "Fatal error committing the changes!\n"
				    "Configuration files left in unknown state.\n");
				return 3;
			}
			*/
		}
		optreset = 1;
		optind = 0;
	}

	error = writeconf(conf);

	if(!quiet && error == 1) {
		fprintf(stderr, "Error committing the changes!\n");
		return 2;
	}
	if(!quiet && error == -1) {
		fprintf(stderr, "Fatal error committing the changes!\n"
		    "Configuration files left in unknown state.\n");
		return 3;
	}
	return 0;
}

void usage(void)
{
	fprintf(stderr,
	    "Usage:\n"
	    "chconf {-q | -f file | -R | -L | -l level "
	    "| -u variable | variable value}\n"
	    "\t-f file\tRead configuration file <file>\n"
	    "\t-FL\tRead loader.conf standard files\n"
	    "\t-FR\tRead rc.conf standard files\n"
	    "\t-l level\tModify level <level> (defaults to 1)\n"
	    "\t-q\tQuiet mode\n");
	exit(1);
}
