#include <stdio.h>
#include <sys/errno.h>
#include "libconf.h"

int printvar(struct confvar*, int);
int printvar2(struct confvar*, int);
int printsec(struct confsec*);

int
main(int argc, char** argv)
{
	struct configuration* conf;
	struct confvar* var;
	struct confsec* sec;
	int level;
	char** p;

	p = argv;

	if(argc < 2) {
		fprintf(stderr, "Usage:\n%s file {file}\n", p[0]);
		return 1;
	}

	conf = createconf(1);

	for(p++; *p; p++)
		readconf(conf, *p);

	printf("Variables:\n");
	for(var = getfirstvar(conf); var; var = getnextvar(var))
		printvar(var, argc-1);

	printf("\nVariables per level:\n");
	for(level = 0; level < argc-1; level++) {
		printf("\nLevel %d\n", level);
		for(var = getfirstvarlevel(conf, level); var; var = getnextvar(var))
			printvar2(var, level);
	}

	printf("\nSections:\n");
	for(sec = getfirstsection(conf); sec; sec = getnextsec(sec)) {
		printf("Configuration file: %s\n", argv[level+1]);
		printsec(sec);
		level = getsecconflevel(sec);
		for(var = getfirstvarsec(sec); var; var = getnextvar(var))
			printvar2(var, level);
		printf("\n");
	}
	

	return 0;
}

int printvar(struct confvar* var, int levels)
{
	int level;

	if(getvarsec(var))
		printf("Section %s\n", getsecname(getvarsec(var)));
	else
		printf("No Section\n");
	printf("%s\t%s(%d)\t%s\n", getvarname(var), getvarvalue(var), getvarlevel(var), getvardesc(var));
	for(level = 0; level < levels; level++) {
		if(varcommented(var, level))
			printf("#");
		if(getvarlevel(var) == level)
			printf("!");
		printf("\t%s(%d)\t%s\n", getvarvaluelevel(var,level), level, getvardesclevel(var,level));
	}
	return 0;
}

int printvar2(struct confvar* var, int level)
{
	if(varcommented(var, level))
		printf("#");
	printf("%s=%s(%d)\t%s\n", getvarname(var), getvarvaluelevel(var, level), getvarlevel(var), getvardesc(var));

	return 0;
}

int printsec(struct confsec* sec)
{
	int i;

	switch(getseclevel(sec)) {
	    case 0:
		for(i = 0; i < 7 + strlen(getsecname(sec)); i++)
			printf("#");
		printf("\n### %s###\n", getsecname(sec));
		for(i = 0; i < getsecdescnlines(sec); i++)
			printf("### %s###\n", getsecdescline(sec,i));
		for(i = 0; i < 7 + strlen(getsecname(sec)); i++)
			printf("#");
		printf("\n");
		break;
	    case 1:
		printf("### %s###\n", getsecname(sec));
		for(i = 0; i < getsecdescnlines(sec); i++)
			printf("### %s###\n", getsecdescline(sec,i));
		break;
	    default:
		printf("#");
		printf(" %s\n", getsecname(sec));
		for(i = 0; i < getsecdescnlines(sec); i++) {
			printf("#");
			printf(" %s\n", getsecdescline(sec,i));
		}
		break;
	}
	return 0;
}



